; This is a sample XLISP program.
; It implements a simple form of programmable turtle

; To run it:

;	(load "pt.lsp")

; This should cause the screen to be cleared and two turtles to appear.
; They should each execute their simple programs and then the prompt
; should return.  Look at the code to see how all of this works.

; Get some more memory
(expand 1)

; ::::::::::::
; :: Turtle ::
; ::::::::::::

; Define "Turtle" class
(setq Turtle (Class :new '(xpos ypos)))

; Answer ":isnew" by initing a position and char and displaying.
(Turtle :answer :isnew '(x y) '(
    (setq xpos x)
    (setq ypos y)
    (self :display)
    self))

; Message ":display" prints its char at its current position
(Turtle :answer :display '() '(
    (moveto xpos ypos)
    (lineto xpos ypos)
    self))

; Message ":goto" goes to a new place after clearing old one
(Turtle :answer :goto '(x y) '(
    (moveto xpos ypos)
    (setq xpos x)
    (setq ypos y)
    (lineto xpos ypos)
    self))

; Message ":up" moves up
(Turtle :answer :up '() '(
    (self :goto xpos (- ypos 10))))

; Message ":down" moves down
(Turtle :answer :down '() '(
    (self :goto xpos (+ ypos 10))))

; Message ":right" moves right
(Turtle :answer :right '() '(
    (self :goto (+ xpos 10) ypos)))

; Message ":left" moves left
(Turtle :answer :left '() '(
    (self :goto (- xpos 10) ypos)))


; :::::::::::::
; :: PTurtle ::
; :::::::::::::

; Define "DPurtle" programable turtle class
(setq PTurtle (Class :new '(prog pc) '() Turtle))

; Message ":program" stores a program
(PTurtle :answer :program '(p) '(
    (setq prog p)
    (setq pc prog)
    self))

; Message ":step" executes a single program step
(PTurtle :answer :step '() '(
    (if (null pc)
	(setq pc prog))
    (if pc
	(progn (self (car pc))
	       (setq pc (cdr pc))))
    self))

; Message ":step:" steps each turtle program n times
(PTurtle :answer :step: '(n) '(
    (dotimes (x n) (self :step))
    self))


; ::::::::::::::
; :: PTurtles ::
; ::::::::::::::

; Define "PTurtles" class
(setq PTurtles (Class :new '(turtles)))

; Message ":make" makes a programable turtle and adds it to the collection
(PTurtles :answer :make '(x y &aux newturtle) '(
    (setq newturtle (PTurtle :new x y))
    (setq turtles (cons newturtle turtles))
    newturtle))

; Message ":step" steps each turtle program once
(PTurtles :answer :step '() '(
    (mapcar #'(lambda (turtle) (turtle :step)) turtles)
    self))

; Message ":step:" steps each turtle program n times
(PTurtles :answer :step: '(n) '(
    (dotimes (x n) (self :step))
    self))


; Create some programmable turtles
(setq turtles (PTurtles :new))
(setq t1 (turtles :make 200 100))
(setq t2 (turtles :make 210 100))
(t1 :program '(:left :left :up :right :up))
(t2 :program '(:right :right :down :left :down))
(show-graphics)
(turtles :step: 10)
